@@ -8,7 +8,7 @@
  *		NET-3 Networking Distribution for the LINUX operating
  *		system.
  *
- * Version:	arp 1.82 (1997-09-25)
+ * Version:	arp 1.83 (1998-02-13)
  *
  * Maintainer:	Bernd 'eckes' Eckenfels, <net-tools@lina.inka.de>
  *
@@ -39,6 +39,8 @@
  *					Typo fix (Debian Bug#5728 Giuliano Procida)
  *970803 {1.81} Bernd Eckenfels :	removed junk comment line 1
  *970925 {1.82} Bernd Eckenfels	:	include fix for libc6
+ *980213 (1.83) Phil Blundell:		set ATF_COM on new entries
+ *980629 (1.84) Arnaldo Carvalho de Melo: gettext instead of catgets
  *
  *
  *		This program is free software; you can redistribute it
@@ -66,7 +68,7 @@
 #include "pathnames.h"
 #include "version.h"
 #include "config.h"
-#include "net-locale.h"
+#include "intl.h"
 
 #define DFLT_AF	"inet"
 #define DFLT_HW	"ether"
@@ -74,9 +76,8 @@
 #define FEATURE_ARP
 #include "lib/net-features.h"
 
-
 char *Release = RELEASE,
-     *Version = "arp 1.82 (1997-09-25)";
+     *Version = "arp 1.84 (1998-06-29)";
 
 int opt_n = 0;				/* do not resolve addresses	*/
 int opt_N = 0;				/* use symbolic names           */
@@ -99,18 +100,17 @@ arp_del(char **args)
   struct arpreq req;
   struct sockaddr sa;
   int flags=0;
-#if HAVE_NEW_SIOCSARP
-  struct arpreq_old old_req;
-#endif
+  int err;
 
   memset((char *) &req, 0, sizeof(req));
 
   /* Resolve the host name. */
   if (*args == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_hostname, "arp: need host name\n"));
+	fprintf(stderr, _("arp: need host name\n"));
 	return(-1);
   }
-  strcpy(host, *args);
+  host[(sizeof host)-1] = 0;
+  strncpy(host, *args, (sizeof host)-1);
   if (ap->input(0, host, &sa) < 0) {
 	ap->herror(host);
 	return(-1);
@@ -190,71 +190,38 @@ arp_del(char **args)
   }
   if (flags == 0)
   	flags = 3;
-  	
-#if HAVE_NEW_SIOCSARP
+
   strcpy(req.arp_dev,device);
 
+  err = -1;
+
   /* Call the kernel. */
   if (flags & 2) {
   	if (opt_v)  fprintf(stderr,"arp: SIOCDARP(nopub)\n");
-  	if (ioctl(sockfd, SIOCDARP, &req) < 0) {
-		if (errno == EINVAL) {
-			if (opt_v)  fprintf(stderr,"arp: OLD_SIOCDARP(priv)\n");
-			memcpy((char *)&old_req,(char *)&req,sizeof(old_req));
-			if (ioctl(sockfd, OLD_SIOCDARP, &old_req) < 0) {
-				if (errno != ENXIO) {
-					perror("OLD_SIOCSARP(priv)");
-					return(-1);
-				}
-			} else {
-				return(0);
-			}
-		}
+  	if ((err = ioctl(sockfd, SIOCDARP, &req) < 0)) {
 		if (errno == ENXIO) {
 			if (flags & 1)
 				goto nopub;
-			printf(NLS_CATGETS(catfd, arpSet, arp_no_arp, 
-				"No ARP entry for %s\n"), host);
+			printf(_("No ARP entry for %s\n"), host);
 			return(-1);
 		}
 		perror("SIOCDARP(priv)");
 		return(-1);
   	}
   }
-  if (flags & 1) {
+  if ((flags & 1) && (err)) {
 nopub:
   	req.arp_flags |= ATF_PUBL;
   	if (opt_v)  fprintf(stderr,"arp: SIOCDARP(pub)\n");
   	if (ioctl(sockfd, SIOCDARP, &req) < 0) {
-		if (errno == EINVAL) {
-			if (opt_v)  fprintf(stderr,"arp: OLD_SIOCDARP(pub)\n");
-			memcpy((char *)&old_req,(char *)&req,sizeof(old_req));
-			if (ioctl(sockfd, OLD_SIOCDARP, &old_req) < 0) {
-				if (errno != ENXIO) {
-					perror("OLD_SIOCSARP(pub)");
-					return(-1);
-				}
-			} else {
-				return(0);
-			}
-		}
 		if (errno == ENXIO) {
-			printf(NLS_CATGETS(catfd, arpSet, arp_no_arp, 
-				"No ARP entry for %s\n"), host);
+			printf(_("No ARP entry for %s\n"), host);
 			return(-1);
 		}
 		perror("SIOCDARP(pub)");
 		return(-1);
   	}
   }
-#else
-  /* Call the kernel. */
-  if (opt_v)  fprintf(stderr,"arp: old_SIOCDARP()\n");
-  if (ioctl(sockfd, SIOCDARP, &req) < 0) {
-	perror("SIOCDARP");
-	return(-1);
-  }
-#endif
 
   return(0);
 }
@@ -292,9 +259,6 @@ arp_set(char **args)
 {
   char host[128];
   struct arpreq req;
-#if HAVE_NEW_SIOCSARP
-  struct arpreq_old old_req;
-#endif
   struct sockaddr sa;
   int flags;
 
@@ -302,10 +266,11 @@ arp_set(char **args)
 
   /* Resolve the host name. */
   if (*args == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_hostname, "arp: need host name\n"));
+	fprintf(stderr, _("arp: need host name\n"));
 	return(-1);
   }
-  strcpy(host, *args++);
+  host[(sizeof host)-1] = 0; 
+  strncpy(host, *args++, (sizeof host)-1);
   if (ap->input(0, host, &sa) < 0) {
 	ap->herror(host);
 	return(-1);
@@ -316,7 +281,7 @@ arp_set(char **args)
 
   /* Fetch the hardware address. */
   if (*args == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_need_hw, "arp: need hardware address\n"));
+	fprintf(stderr, _("arp: need hardware address\n"));
 	return(-1);
   }
 
@@ -325,13 +290,13 @@ arp_set(char **args)
       return(-1);
   } else {
     if (hw->input(*args++, &req.arp_ha) < 0) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_invalidhw, "arp: invalid hardware address\n"));
+	fprintf(stderr, _("arp: invalid hardware address\n"));
 	return(-1);
     }
   }
   
   /* Check out any modifiers. */
-  flags = ATF_PERM;
+  flags = ATF_PERM | ATF_COM;
   while (*args != NULL) {
 	if (! strcmp(*args, "temp")) {
 		flags &= ~ATF_PERM;
@@ -401,9 +366,7 @@ arp_set(char **args)
   /* Fill in the remainder of the request. */
   req.arp_flags = flags;
 
-#if HAVE_NEW_SIOCSARP
   strcpy(req.arp_dev,device);
-  memcpy((char *)&old_req,(char *)&req,sizeof(old_req));
 
   /* Call the kernel. */
   if (opt_v)  fprintf(stderr,"arp: SIOCSARP()\n");
@@ -412,30 +375,7 @@ arp_set(char **args)
 		perror("SIOCSARP");
 		return(-1);
 	}
-	if (opt_v)  fprintf(stderr,"arp: OLD_SIOCSARP()\n");
-	if (ioctl(sockfd, OLD_SIOCSARP, &old_req) < 0) {
-		if (errno != EINVAL) {
-			perror("OLD_SIOCSARP");
-			return(-1);
-		}
-		perror("SIOCSARP and OLD_SIOCSARP");
-		if (flags & ATF_PUBL)
-			fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_einv_pub, 
-					"Probably route to destination points to interface. See arp(8)\n"));
-		else
-			fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_einv_nopub, 
-					"No ARP Support or route to destination points to other interface. See arp(8)\n"));
-		return(-1);
-	}
   }
-#else
-  /* Call the kernel. */
-  if (opt_v)  fprintf(stderr,"arp: old_SIOCSARP()\n");
-  if (ioctl(sockfd, SIOCSARP, &req) < 0) {
-	perror("SIOCSARP");
-	return(-1);
-  }
-#endif
 
   return(0);
 }
@@ -451,7 +391,7 @@ arp_file(char *name)
   FILE *fp;
 
   if ((fp = fopen(name, "r")) == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_cant_open, "arp: cannot open etherfile %s !\n"), name);
+	fprintf(stderr, _("arp: cannot open etherfile %s !\n"), name);
 	return(-1);
   }
 
@@ -465,17 +405,14 @@ arp_file(char *name)
 
 	argc = getargs(buff, args);
 	if (argc < 2) {
-		fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_formaterr, 
-					    "arp: format error on line %u of etherfile %s !\n"),
+		fprintf(stderr, _("arp: format error on line %u of etherfile %s !\n"),
 			linenr, name);
 		continue;
 	}
 
-	if (arp_set(args) != 0) {
-		fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_cant_set,
-					    "arp: cannot set entry on line %u of etherfile %s !\n"),
+	if (arp_set(args) != 0)
+		fprintf(stderr, _("arp: cannot set entry on line %u of etherfile %s !\n"),
 			linenr, name);
-	}
   }
 
   (void) fclose(fp);
@@ -496,8 +433,7 @@ arp_disp_2(char *name,int type,int arp_flags,char *hwa,char *mask,char *dev)
     xhw = get_hwtype(DFLT_HW);
     
   if (title++ == 0) {
-    printf(NLS_CATGETS(catfd, arpSet, arp_address,
-		       "Address\t\t\tHWtype\tHWaddress\t    Flags Mask\t\t  Iface\n"));
+    printf(_("Address\t\t\tHWtype\tHWaddress\t    Flags Mask\t\t  Iface\n"));
   }
   /* Setup the flags. */
   flags[0] = '\0';
@@ -588,7 +524,8 @@ arp_show(char *name)
   
   if (name != NULL) {
   	/* Resolve the host name. */
-  	strcpy(host, name);
+	  host[(sizeof host)-1] = 0;
+  	strncpy(host, name, (sizeof host)-1);
   	if (ap->input(0, host, &sa) < 0) {
 		ap->herror(host);
 		return(-1);
@@ -609,7 +546,7 @@ arp_show(char *name)
 	/* Read the ARP cache entries. */
 	for(;fgets(line,sizeof(line),fp);)
 	{
-		num=sscanf(line,"%s 0x%x 0x%x %s %s %s\n",
+		num=sscanf(line,"%s 0x%x 0x%x %100s %100s %100s\n",
 		                               ip,&type,&flags,hwa,mask,dev);
 		if(num<4)
 			break;
@@ -647,15 +584,13 @@ arp_show(char *name)
 	}
   }
   if (opt_v)
-	printf(NLS_CATGETS(catfd, arpSet, arp_sum,
-		"Entries: %d\tSkipped: %d\tFound: %d\n"),entries,entries-showed,showed);
+	printf(_("Entries: %d\tSkipped: %d\tFound: %d\n"),entries,entries-showed,showed);
   
   if (!showed) {
   	if (host[0] && !opt_a)
   		printf("%s (%s) -- no entry\n", name, host);
   	else if (hw_set || host[0] || device[0]) {
-		printf(NLS_CATGETS(catfd, arpSet, arp_none,
-			"arp: in %d entries no match found.\n"),entries);
+		printf(_("arp: in %d entries no match found.\n"),entries);
  	}
   }
   (void) fclose(fp);
@@ -666,26 +601,18 @@ static void
 version(void)
 {
   fprintf(stderr, "%s\n%s\n%s\n",Release,Version,Features);
-  NLS_CATCLOSE(catfd)
   exit(-1);
 }
 
 static void
 usage(void)
 {
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage1,
-	"Usage: arp [-vn] [-H type] [-i if] -a [hostname]\n"));
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage2,
-	"       arp [-v] [-i if] -d hostname [pub][nopub]\n"));
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage3,
-	"       arp [-v] [-H type] [-i if] -s  hostname hw_addr [temp][nopub]\n"));
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage4,
-	"       arp [-v] [-H type] [-i if] -s  hostname hw_addr [netmask nm] pub\n"));
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage5,
-	"       arp [-v] [-H type] [-i if] -Ds hostname if [netmask nm] pub\n"));
-  fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_usage6,
-	"       arp [-vnD] [-H type] [-i if] -f filename\n"));
-  NLS_CATCLOSE(catfd)
+  fprintf(stderr, _("Usage: arp [-vn] [-H type] [-i if] -a [hostname]\n"));
+  fprintf(stderr, _("       arp [-v] [-i if] -d hostname [pub][nopub]\n"));
+  fprintf(stderr, _("       arp [-v] [-H type] [-i if] -s  hostname hw_addr [temp][nopub]\n"));
+  fprintf(stderr, _("       arp [-v] [-H type] [-i if] -s  hostname hw_addr [netmask nm] pub\n"));
+  fprintf(stderr, _("       arp [-v] [-H type] [-i if] -Ds hostname if [netmask nm] pub\n"));
+  fprintf(stderr, _("       arp [-vnD] [-H type] [-i if] -f filename\n"));
   exit(-1);
 }
 
@@ -711,22 +638,18 @@ main(int argc, char **argv)
 	{NULL,		0,	0,	0}
   };	
              
-#if NLS
-  setlocale (LC_MESSAGES, "");
-  catfd = catopen ("nettools", MCLoadBySet);
+#if I18N
+  bindtextdomain("net-tools", "/usr/share/locale");
+  textdomain("net-tools");
 #endif
 
   /* Initialize variables... */
   if ((hw = get_hwtype(DFLT_HW)) == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_hw_not_supp,
-				    "%s: hardware type not supported!\n"), DFLT_HW);
-	NLS_CATCLOSE(catfd)
+	fprintf(stderr, _("%s: hardware type not supported!\n"), DFLT_HW);
 	return(-1);
   }
   if ((ap = get_aftype(DFLT_AF)) == NULL) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_fam_not_supp,
-				    "%s: address family not supported!\n"), DFLT_AF);
-	NLS_CATCLOSE(catfd)
+	fprintf(stderr, _("%s: address family not supported!\n"), DFLT_AF);
 	return(-1);
   }
   what = 0;
@@ -770,10 +693,8 @@ main(int argc, char **argv)
 	case 'p':
 		ap = get_aftype(optarg);
 		if (ap == NULL) {
-			fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_unkn_addr,
-						    "arp: %s: unknown address family.\n"),
+			fprintf(stderr, _("arp: %s: unknown address family.\n"),
 				optarg);
-			NLS_CATCLOSE(catfd)
 			exit(-1);
 		}
 		break;
@@ -781,10 +702,8 @@ main(int argc, char **argv)
 	case 't':
 		hw = get_hwtype(optarg);
 		if (hw == NULL) {
-			fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_unkn_hw,
-						    "arp: %s: unknown hardware type.\n"),
+			fprintf(stderr, _("arp: %s: unknown hardware type.\n"),
 				optarg);
-			NLS_CATCLOSE(catfd)
 			exit(-1);
 		}
 		hw_set = 1;
@@ -803,23 +722,18 @@ main(int argc, char **argv)
   }
 
   if (ap->af != AF_INET) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_wrong_af,
-				    "arp: %s: kernel only supports 'inet'.\n"),
+	fprintf(stderr, _("arp: %s: kernel only supports 'inet'.\n"),
 			ap->name);
-	NLS_CATCLOSE(catfd)
 	exit(-1);
   }
   if (hw->alen <= 0) {
-	fprintf(stderr, NLS_CATGETS(catfd, arpSet, arp_wrong_hw,
-				    "arp: %s: hardware type without ARP support.\n"),
+	fprintf(stderr, _("arp: %s: hardware type without ARP support.\n"),
 			hw->name);
-	NLS_CATCLOSE(catfd)
 	exit(-1);
   }
   if ((sockfd = socket(AF_INET,SOCK_DGRAM,0)) <0)
   {
   	perror("socket");
-	NLS_CATCLOSE(catfd)
   	exit(-1);
   }
 
@@ -850,6 +764,5 @@ main(int argc, char **argv)
 		usage();
   }
 
-  NLS_CATCLOSE(catfd)
   exit(what);
 }
