@@ -3,7 +3,7 @@
  *		that either displays or sets the characteristics of
  *		one or more of the system's networking interfaces.
  *
- * Version:	ifconfig 1.31 (1997-09-25)
+ * Version:	ifconfig 1.34 (1998-06-30) 
  *
  * Author:	Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
  *              and others.  Copyright 1993 MicroWalt Corporation
@@ -13,6 +13,9 @@
  *		Public  License as  published  by  the  Free  Software
  *		Foundation;  either  version 2 of the License, or  (at
  *		your option) any later version.
+ * {1.34} - 19980630 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+ *                     - gettext instead of catgets for i18n
+ *	    10/1998  - Andi Kleen. Use interface list primitives. 	
  */
 
 #include "config.h"
@@ -36,8 +39,13 @@
 /* Ugh.  But libc5 doesn't provide POSIX types.  */
 #include <asm/types.h>
 
+#ifdef HAVE_HWSLIP
+#include <linux/if_slip.h>
+#endif
+
 #if HAVE_AFINET6
 
+#ifndef _LINUX_IN6_H
 /*
  *	This is in linux/include/net/ipv6.h.
  */
@@ -47,6 +55,8 @@ struct in6_ifreq {
   __u32 ifr6_prefixlen;
   unsigned int ifr6_ifindex;
 };
+
+#endif
  
 #define IPV6_ADDR_ANY		0x0000U
 
@@ -67,73 +77,7 @@ struct in6_ifreq {
 
 #endif  /* HAVE_AFINET6 */
 
-#ifndef IFF_PORTSEL
-#define IFF_PORTSEL	0x2000
-#define IFF_AUTOMEDIA	0x4000
-#endif
-
-/* This is from <linux/netdevice.h>. */
-
-struct user_net_device_stats
-{
-  unsigned long	rx_packets;	/* total packets received	*/
-  unsigned long	tx_packets;	/* total packets transmitted	*/
-  unsigned long	rx_bytes;	/* total bytes received 	*/
-  unsigned long	tx_bytes;	/* total bytes transmitted	*/
-  unsigned long	rx_errors;	/* bad packets received		*/
-  unsigned long	tx_errors;	/* packet transmit problems	*/
-  unsigned long	rx_dropped;	/* no space in linux buffers	*/
-  unsigned long	tx_dropped;	/* no space available in linux	*/
-  unsigned long	multicast;	/* multicast packets received	*/
-  unsigned long	collisions;
-
-  /* detailed rx_errors: */
-  unsigned long	rx_length_errors;
-  unsigned long	rx_over_errors;	/* receiver ring buff overflow	*/
-  unsigned long	rx_crc_errors;	/* recved pkt with crc error	*/
-  unsigned long	rx_frame_errors; /* recv'd frame alignment error */
-  unsigned long	rx_fifo_errors;	/* recv'r fifo overrun		*/
-  unsigned long	rx_missed_errors; /* receiver missed packet	*/
-  /* detailed tx_errors */
-  unsigned long	tx_aborted_errors;
-  unsigned long	tx_carrier_errors;
-  unsigned long	tx_fifo_errors;
-  unsigned long	tx_heartbeat_errors;
-  unsigned long	tx_window_errors;
-
-  /* private to ifconfig tostore version dependend flags */
-  short has_bytes;	/* is rx/tx_bytes valid? */
-  short has_noavail;	/* is "no stat avail"? */
-};
-
-struct interface {
-  char			name[IFNAMSIZ];		/* interface name	 */
-  short			type;			/* if type		 */
-  short			flags;			/* various flags	 */
-  int			metric;			/* routing metric	 */
-  int			mtu;			/* MTU value		 */
-  struct ifmap		map;			/* hardware setup	 */
-  struct sockaddr	addr;			/* IP address		 */
-  struct sockaddr	dstaddr;		/* P-P IP address	 */
-  struct sockaddr	broadaddr;		/* IP broadcast address	 */
-  struct sockaddr	netmask;		/* IP network mask	 */
-  struct sockaddr	ipxaddr_bb;		/* IPX network address   */
-  struct sockaddr	ipxaddr_sn;		/* IPX network address   */
-  struct sockaddr	ipxaddr_e3;		/* IPX network address   */
-  struct sockaddr	ipxaddr_e2;		/* IPX network address   */
-  struct sockaddr	ddpaddr;		/* Appletalk DDP address */
-  int			has_ip;
-  int			has_ipx_bb;
-  int			has_ipx_sn;
-  int			has_ipx_e3;
-  int			has_ipx_e2;
-  int			has_ax25;
-  int			has_rose;
-  int			has_ddp;
-  char			hwaddr[32];		/* HW address		 */
-  struct user_net_device_stats stats;		/* statistics		 */
-};
-
+#ifdef IFF_PORTSEL
 static const char *if_port_text[][4] = {
   /* Keep in step with <linux/netdevice.h> */
   { "unknown", NULL , NULL, NULL },
@@ -145,53 +89,29 @@ static const char *if_port_text[][4] = {
   { "100baseFX", NULL, NULL, NULL },
   { NULL, NULL, NULL, NULL },
 };
-
-/* Not all kernels have these new ioctls. */
-#ifndef SIOGIFNAME
-#define SIOGIFNAME	0x8934		/* if_index -> name mapping	*/
-#endif
-#ifndef SIOCGIFCOUNT
-#define SIOCGIFCOUNT	0x8935		/* get number of interfaces	*/
-#endif
-#ifndef SIOCDIFADDR
-#define SIOCDIFADDR	0x8936		/* delete PA address		*/
 #endif
 
 #if HAVE_AFIPX
+#if (__GLIBC__ > 2) || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 1)
+#include <netipx/ipx.h>
+#else
 #include "ipx.h"
 #endif
+#endif
 #include "net-support.h"
 #include "pathnames.h"
 #include "version.h"
-#include "net-locale.h"
+#include "../intl.h"
+#include "interface.h"
+#include "sockets.h"
 
 char *Release = RELEASE,
-     *Version = "ifconfig 1.30 (1997-09-29)";
-
+     *Version = "ifconfig 1.36 (1998-10-31)";
 
 int opt_a = 0;				/* show all interfaces		*/
 int opt_i = 0;				/* show the statistics		*/
 int opt_v = 0;				/* debugging output flag	*/
 
-int skfd = -1;				/* generic raw socket desc.	*/
-#if HAVE_AFIPX
-int ipx_sock = -1;			/* IPX socket			*/
-#endif
-#if HAVE_AFAX25
-int ax25_sock = -1;			/* AX.25 socket			*/
-#endif
-#if HAVE_AFROSE
-int rose_sock = -1;			/* Rose socket			*/
-#endif
-#if HAVE_AFINET
-int inet_sock = -1;			/* INET socket			*/
-#endif
-#if HAVE_AFINET6
-int inet6_sock = -1;			/* INET6 socket			*/
-#endif
-#if HAVE_AFATALK
-int ddp_sock = -1;			/* Appletalk DDP socket		*/
-#endif
 int addr_family = 0;			/* currently selected AF	*/
 
 
@@ -201,11 +121,19 @@ ife_print(struct interface *ptr)
   struct aftype *ap;
   struct hwtype *hw;
   int hf;
-  char *dispname=NLS_CATSAVE (catfd, ifconfigSet, ifconfig_over, "over");
-  static struct aftype *ipxtype=NULL, *ddptype=NULL;
+  int can_compress = 0;
+#if HAVE_AFIPX
+  static struct aftype *ipxtype=NULL;
+#endif
+#if HAVE_AFECONET
+  static struct aftype *ectype = NULL;
+#endif
+#if HAVE_AFATALK
+   static struct aftype *ddptype = NULL;
+#endif
 #if HAVE_AFINET6
   FILE *f;
-  char addr6[40], devname[10];
+  char addr6[40], devname[20];
   struct sockaddr_in6 sap;
   int plen, scope, dad_status, if_idx;
   extern struct aftype inet6_aftype;
@@ -220,51 +148,46 @@ ife_print(struct interface *ptr)
 
   hf=ptr->type;
 
-  if(strncmp(ptr->name,"lo",2)==0)
+  if (strncmp(ptr->name, "lo", 2) == 0)
   	hf=255;
   	
-  if(hf==ARPHRD_CSLIP || hf==ARPHRD_CSLIP6)
-  {
-#if NLS
-    /* NLS must free dispname */
-    free (dispname);
-#endif
-    /* Overrun got reused: BAD - fix later */
-    dispname=NLS_CATSAVE (catfd, ifconfigSet, ifconfig_compress, "comp");
-  }
+  if (hf==ARPHRD_CSLIP || hf==ARPHRD_CSLIP6)
+    can_compress = 1;
   
   hw = get_hwntype(hf);
   if (hw == NULL) hw = get_hwntype(-1);
 
-  printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_link, 
-		     "%-8.8s  Link encap:%s  "), ptr->name, hw->title);
-  /* Don't print the hardware address for ATM if it's null is in hw->print() */
-  if (hw->print != NULL)
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_hw, "HWaddr %s  ")
-	   , hw->print(ptr->hwaddr));
+  printf(_("%-8.8s  Link encap:%s  "), ptr->name, hw->title);
+  /* Don't print the hardware address for ATM or Ash if it's null. */
+  if (hw->sprint != NULL && ((strncmp(ptr->name, "atm", 3) &&
+			      strncmp(ptr->name, "ash", 3)) || 
+   (ptr->hwaddr[0] || ptr->hwaddr[1] || ptr->hwaddr[2] || ptr->hwaddr[3] || 
+   ptr->hwaddr[4] || ptr->hwaddr[5]))) 
+    printf(_("HWaddr %s  "), hw->print(ptr->hwaddr));
+#ifdef IFF_PORTSEL
   if (ptr->flags & IFF_PORTSEL) 
-    printf("Media:%s%s", if_port_text[ptr->map.port][0], 
-	   (ptr->flags & IFF_AUTOMEDIA)?"(auto)":"");
+    printf(_("Media:%s%s"), if_port_text[ptr->map.port][0], 
+	   (ptr->flags & IFF_AUTOMEDIA)?_("(auto)"):"");
+#endif
   printf("\n");
 #if HAVE_AFINET6  
   if (ap->af != AF_INET6) {
 #endif
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_adr,
-		       "          %s addr:%s"), ap->name,
+    printf(_("          %s addr:%s "), ap->name,
 	   ap->sprint(&ptr->addr, 1));
     if (ptr->flags & IFF_POINTOPOINT) {
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_pap, "  P-t-P:%s  "),
-	     ap->sprint(&ptr->dstaddr, 1));
-    } else {
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_bcast, "  Bcast:%s  "),
-	     ap->sprint(&ptr->broadaddr, 1));
+      printf(_(" P-t-P:%s "), ap->sprint(&ptr->dstaddr, 1));
+    } 
+    if (ptr->flags & IFF_BROADCAST) {
+      printf(_(" Bcast:%s "), ap->sprint(&ptr->broadaddr, 1));
     }
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mask, "Mask:%s\n"),
-	   ap->sprint(&ptr->netmask, 1));
+    printf(_(" Mask:%s\n"), ap->sprint(&ptr->netmask, 1));
 #if HAVE_AFINET6
   }
+  /* FIXME: should be integrated into interface.c.   */ 
+
   if ((f = fopen(_PATH_PROCNET_IFINET6, "r")) != NULL) {
-    while(fscanf(f, "%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %s\n",
+    while(fscanf(f, "%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %20s\n",
 		 addr6p[0], addr6p[1], addr6p[2], addr6p[3],
 		 addr6p[4], addr6p[5], addr6p[6], addr6p[7],
 		 &if_idx, &plen, &scope, &dad_status, devname) != EOF) {
@@ -273,18 +196,16 @@ ife_print(struct interface *ptr)
 		addr6p[0], addr6p[1], addr6p[2], addr6p[3],
 		addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
 	inet6_aftype.input(1, addr6, (struct sockaddr *)&sap);
-	printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_adr6,
-			   "          inet6 addr: %s/%d"),
+	printf(_("          inet6 addr: %s/%d"),
 	       inet6_aftype.sprint((struct sockaddr *)&sap, 1), plen);
-	printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_scope,
-				   " Scope:"));
+	printf(_(" Scope:"));
 	switch (scope) {
-	case 0: printf("Global"); break;
-	case IPV6_ADDR_LINKLOCAL: printf("Link"); break;
-	case IPV6_ADDR_SITELOCAL: printf("Site"); break;
-	case IPV6_ADDR_COMPATv4: printf("Compat"); break;
-	case IPV6_ADDR_LOOPBACK: printf("Host"); break;
-	default: printf("Unknown");
+	case 0: printf(_("Global")); break;
+	case IPV6_ADDR_LINKLOCAL: printf(_("Link")); break;
+	case IPV6_ADDR_SITELOCAL: printf(_("Site")); break;
+	case IPV6_ADDR_COMPATv4: printf(_("Compat")); break;
+	case IPV6_ADDR_LOOPBACK: printf(_("Host")); break;
+	default: printf(_("Unknown"));
 	}
 	printf("\n");
       }
@@ -293,360 +214,152 @@ ife_print(struct interface *ptr)
   }
 #endif
   
+#if HAVE_AFIPX
   if (ipxtype==NULL)
     ipxtype=get_afntype(AF_IPX);
 
   if (ipxtype!=NULL) {
     if(ptr->has_ipx_bb)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_etherII,
-			 "          IPX/Ethernet II addr:%s\n"),
+      printf(_("          IPX/Ethernet II addr:%s\n"),
 	     ipxtype->sprint(&ptr->ipxaddr_bb,1));
     if(ptr->has_ipx_sn)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_SNAP,
-			 "          IPX/Ethernet SNAP addr:%s\n"),
+      printf(_("          IPX/Ethernet SNAP addr:%s\n"),
 	     ipxtype->sprint(&ptr->ipxaddr_sn,1));
     if(ptr->has_ipx_e2)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_8022,
-			 "          IPX/Ethernet 802.2 addr:%s\n"),
+      printf(_("          IPX/Ethernet 802.2 addr:%s\n"),
 	     ipxtype->sprint(&ptr->ipxaddr_e2,1));
     if(ptr->has_ipx_e3)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_8023,
-			 "          IPX/Ethernet 802.3 addr:%s\n"),
+      printf(_("          IPX/Ethernet 802.3 addr:%s\n"),
 	     ipxtype->sprint(&ptr->ipxaddr_e3,1));
   }
+#endif
+
+#if HAVE_AFATALK
   if (ddptype==NULL)
     ddptype=get_afntype(AF_APPLETALK);
   if (ddptype!=NULL) {
     if (ptr->has_ddp)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_talk,
-			 "          EtherTalk Phase 2 addr:%s\n"),
-	     ddptype->sprint(&ptr->ddpaddr,1));
+      printf(_("          EtherTalk Phase 2 addr:%s\n"), ddptype->sprint(&ptr->ddpaddr,1));
   }
-  printf("          ");
-  if (ptr->flags == 0) printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_noflags,
-					  "[NO FLAGS] "));
-  if (ptr->flags & IFF_UP) printf("UP ");
-  if (ptr->flags & IFF_BROADCAST) printf("BROADCAST ");
-  if (ptr->flags & IFF_DEBUG) printf("DEBUG ");
-  if (ptr->flags & IFF_LOOPBACK) printf("LOOPBACK ");
-  if (ptr->flags & IFF_POINTOPOINT) printf("POINTOPOINT ");
-  if (ptr->flags & IFF_NOTRAILERS) printf("NOTRAILERS ");
-  if (ptr->flags & IFF_RUNNING) printf("RUNNING ");
-  if (ptr->flags & IFF_NOARP) printf("NOARP ");
-  if (ptr->flags & IFF_PROMISC) printf("PROMISC ");
-  if (ptr->flags & IFF_ALLMULTI) printf("ALLMULTI ");
-  if (ptr->flags & IFF_SLAVE) printf("SLAVE ");
-  if (ptr->flags & IFF_MASTER) printf("MASTER ");
-  if (ptr->flags & IFF_MULTICAST) printf("MULTICAST ");
-  printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mtu, " MTU:%d  Metric:%d\n"),
-	 ptr->mtu, ptr->metric?ptr->metric:1);
+#endif
 
+#if HAVE_AFECONET
+  if (ectype == NULL)
+    ectype = get_afntype(AF_ECONET);
+  if (ectype != NULL) {
+    if (ptr->has_econet)
+      printf(_("          econet addr:%s\n"), ectype->sprint(&ptr->ecaddr,1));
+  }
+#endif
+
+  printf("          ");
+  if (ptr->flags == 0) printf(_("[NO FLAGS] "));
+  if (ptr->flags & IFF_UP) printf(_("UP "));
+  if (ptr->flags & IFF_BROADCAST) printf(_("BROADCAST "));
+  if (ptr->flags & IFF_DEBUG) printf(_("DEBUG "));
+  if (ptr->flags & IFF_LOOPBACK) printf(_("LOOPBACK "));
+  if (ptr->flags & IFF_POINTOPOINT) printf(_("POINTOPOINT "));
+  if (ptr->flags & IFF_NOTRAILERS) printf(_("NOTRAILERS "));
+  if (ptr->flags & IFF_RUNNING) printf(_("RUNNING "));
+  if (ptr->flags & IFF_NOARP) printf(_("NOARP "));
+  if (ptr->flags & IFF_PROMISC) printf(_("PROMISC "));
+  if (ptr->flags & IFF_ALLMULTI) printf(_("ALLMULTI "));
+  if (ptr->flags & IFF_SLAVE) printf(_("SLAVE "));
+  if (ptr->flags & IFF_MASTER) printf(_("MASTER "));
+  if (ptr->flags & IFF_MULTICAST) printf(_("MULTICAST "));
+#ifdef HAVE_DYNAMIC
+  if (ptr->flags & IFF_DYNAMIC) printf(_("DYNAMIC ")); 
+#endif  
+
+  printf(_(" MTU:%d  Metric:%d"),
+	 ptr->mtu, ptr->metric?ptr->metric:1);
+#ifdef SIOCSKEEPALIVE
+  if (ptr->outfill || ptr->keepalive)
+    printf(_("  Outfill:%d  Keepalive:%d"),
+	      ptr->outfill, ptr->keepalive);
+#endif
+  printf("\n");
 
   /* If needed, display the interface statistics. */
+
+  if (ptr->statistics_valid) { 
+	  /* XXX: statistics are currently only printed for the original address,
+	   *	  not for the aliases, although strictly speaking they're shared
+	   *	  by all addresses.
+	   */
   printf("          ");
 
-  if (ptr->stats.has_noavail) {
-  	printf("No statistics available.\n"); /* XXX */
-  } else {
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_rx,
-	"RX packets:%lu error:%lu dropped:%lu %s:%lu frame:%lu\n"),
+  printf(_("RX packets:%lu errors:%lu dropped:%lu overruns:%lu frame:%lu\n"),
 	ptr->stats.rx_packets, ptr->stats.rx_errors,
-	ptr->stats.rx_dropped, dispname, ptr->stats.rx_fifo_errors,
+	ptr->stats.rx_dropped, ptr->stats.rx_fifo_errors,
         ptr->stats.rx_frame_errors);
+  if (can_compress)
+    printf(_("             compressed:%lu\n"), ptr->stats.rx_compressed);
 	 
-    printf("          ");
+  printf("          ");
 
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_tx,
-	"TX packets:%lu error:%lu dropped:%lu %s:%lu carrier:%lu coll:%lu\n"),
+  printf(_("TX packets:%lu errors:%lu dropped:%lu overruns:%lu carrier:%lu\n"),
 	ptr->stats.tx_packets, ptr->stats.tx_errors,
-	ptr->stats.tx_dropped, dispname, ptr->stats.tx_fifo_errors,
-	ptr->stats.tx_carrier_errors, ptr->stats.collisions);
+	ptr->stats.tx_dropped, ptr->stats.tx_fifo_errors,
+	ptr->stats.tx_carrier_errors);
+  printf(_("          collisions:%lu "), ptr->stats.collisions);
+  if (can_compress)
+    printf(_("compressed:%lu "), ptr->stats.tx_compressed);
+  if (ptr->tx_queue_len != -1)
+    printf(_("txqueuelen:%d "), ptr->tx_queue_len);
+  printf("\n"); 
   }
 
-  if (hf<255 && (ptr->map.irq || ptr->map.mem_start || ptr->map.dma || 
+  if ((ptr->map.irq || ptr->map.mem_start || ptr->map.dma || 
 		ptr->map.base_addr)) {
     printf("          ");
     if (ptr->map.irq)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_interrupt,
-			 "Interrupt:%d "), ptr->map.irq);
+      printf(_("Interrupt:%d "), ptr->map.irq);
     if (ptr->map.base_addr>=0x100)      /* Only print devices using it for 
 					  I/O maps */
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_base,
-			 "Base address:0x%x "), ptr->map.base_addr);
+      printf(_("Base address:0x%x "), ptr->map.base_addr);
     if (ptr->map.mem_start) {
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mem, "Memory:%lx-%lx "),
-	     ptr->map.mem_start,ptr->map.mem_end);
+      printf(_("Memory:%lx-%lx "), ptr->map.mem_start,ptr->map.mem_end);
     }
     if (ptr->map.dma)
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_dma, "DMA chan:%x "),
-	     ptr->map.dma);
+      printf(_("DMA chan:%x "), ptr->map.dma);
     printf("\n");
   }
   
   printf("\n");
-
-#if NLS
-  /* NLS must free dispname */
-  free (dispname);
-#endif
 }
 
-
-static void if_getstats(char *ifname, struct interface *ife)
+static int do_if_print(struct interface *ife, void *cookie)
 {
-  FILE *f = fopen(_PATH_PROCNET_DEV, "r");
-  char buf[256];
-  char *bp, *tmp;
-  if (f==NULL)
-    return;
+    int *opt_a = (int *)cookie; 
 
-  if (!fgets(buf,255,f))
-  	{ fclose(f); return; }
-  if (!fgets(buf,255,f))
-  	{ fclose(f); return; }
-
-  ife->stats.has_bytes = strstr(buf, "bytes")?1:0;
-  ife->stats.has_noavail = 0;
-
-  while(fgets(buf,255,f))
-   {
-    bp=buf;
-    while(*bp&&isspace(*bp))
-      bp++;
-    if(strncmp(bp,ifname,strlen(ifname))==0 && bp[strlen(ifname)]==':') {
-      if (!(bp=strchr(bp,':'))) /* We dont want alias interfaces */
-	continue;
-      bp++;
-      if ((tmp=strchr(bp,':')))
-        bp=tmp+1;
-      while(*bp&&isspace(*bp))
-        bp++;
-      if (strncmp(bp, "No", 2)==0) { /* Check if this interface dont have stats */
-  	ife->stats.has_noavail = 1;
-	break;
-      }
-      if (ife->stats.has_bytes) {
-	sscanf(bp,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
-	       &ife->stats.rx_bytes,
-	       &ife->stats.rx_packets,
-	       &ife->stats.rx_errors,
-	       &ife->stats.rx_dropped,
-	       &ife->stats.rx_fifo_errors,
-	       &ife->stats.rx_frame_errors,
-	       
-	       &ife->stats.tx_bytes,
-	       &ife->stats.tx_packets,
-	       &ife->stats.tx_errors,
-	       &ife->stats.tx_dropped,
-	       &ife->stats.tx_fifo_errors,
-	       &ife->stats.collisions,
-	       
-	       &ife->stats.tx_carrier_errors
-	       );
-      } else {
-	sscanf(bp,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
-	       &ife->stats.rx_packets,
-	       &ife->stats.rx_errors,
-	       &ife->stats.rx_dropped,
-	       &ife->stats.rx_fifo_errors,
-	       &ife->stats.rx_frame_errors,
-	       
-	       &ife->stats.tx_packets,
-	       &ife->stats.tx_errors,
-	       &ife->stats.tx_dropped,
-	       &ife->stats.tx_fifo_errors,
-	       &ife->stats.collisions,
-	       
-	       &ife->stats.tx_carrier_errors
-	       );
-	ife->stats.has_noavail = 0;
-      }
-      break;
+    if (if_fetch(ife->name, ife) < 0) {
+	fprintf (stderr, _("%s: error fetching interface information: %s\n\n"), 
+		 ife->name, strerror(errno));
+	return -1;
     }
-  }
-  fclose(f);
-  return;
+    if (!(ife->flags & IFF_UP) && !(*opt_a))
+	return 0;
+    ife_print(ife); 
+    return 0;
 }
 
-/* Support for fetching an IPX address */
-
-#if HAVE_AFIPX
-static int ipx_getaddr(int sock, int ft, struct ifreq *ifr)
+static void if_print(char *ifname)
 {
-	((struct sockaddr_ipx *)&ifr->ifr_addr)->sipx_type=ft;
-	return ioctl(sock, SIOCGIFADDR, ifr);
-}
-#endif
+    struct interface *ife;
 
-/* Fetch the interface configuration from the kernel. */
-static int
-if_fetch(char *ifname, struct interface *ife)
-{
-  struct ifreq ifr;
-
-  memset((char *) ife, 0, sizeof(struct interface));
-  strcpy(ife->name, ifname);
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) return(-1);
-  ife->flags = ifr.ifr_flags;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) 
-    memset(ife->hwaddr, 0, 32);
-  else 
-    memcpy(ife->hwaddr,ifr.ifr_hwaddr.sa_data,8);
-
-  ife->type=ifr.ifr_hwaddr.sa_family;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMETRIC, &ifr) < 0)
-    ife->metric = 0;
-  else 
-    ife->metric = ifr.ifr_metric;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMTU, &ifr) < 0)
-    ife->mtu = 0;
-  else 
-    ife->mtu = ifr.ifr_mtu;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0)
-    memset(&ife->map, 0, sizeof(struct ifmap));
-  else 
-    memcpy(&ife->map,&ifr.ifr_map,sizeof(struct ifmap));
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0)
-    memset(&ife->map, 0, sizeof(struct ifmap));
-  else 
-    ife->map = ifr.ifr_map;
-  
-#if HAVE_AFINET
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFDSTADDR, &ifr) < 0)
-    memset(&ife->dstaddr, 0, sizeof(struct sockaddr));
-  else 
-    ife->dstaddr = ifr.ifr_dstaddr;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFBRDADDR, &ifr) < 0)
-    memset(&ife->broadaddr, 0, sizeof(struct sockaddr));
-  else 
-    ife->broadaddr = ifr.ifr_broadaddr;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFNETMASK, &ifr) < 0)
-    memset(&ife->netmask, 0, sizeof(struct sockaddr));
-  else 
-    ife->netmask = ifr.ifr_netmask;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFADDR, &ifr) < 0) 
-    memset(&ife->addr, 0, sizeof(struct sockaddr));
-  else 
-    ife->addr = ifr.ifr_addr;
-#endif
-  
-#if HAVE_AFATALK
-  /* DDP address maybe ? */
-  strcpy(ifr.ifr_name, ifname);
-  if (ddp_sock >= 0 && ioctl(ddp_sock, SIOCGIFADDR, &ifr) == 0) {
-    ife->ddpaddr=ifr.ifr_addr;
-    ife->has_ddp=1;
-  }
-#endif
-
-#if HAVE_AFIPX  
-  /* Look for IPX addresses with all framing types */
-  strcpy(ifr.ifr_name, ifname);
-  if (ipx_sock >= 0) {
-    if (!ipx_getaddr(ipx_sock, IPX_FRAME_ETHERII, &ifr)) {
-      ife->has_ipx_bb=1;
-      ife->ipxaddr_bb=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if (!ipx_getaddr(ipx_sock, IPX_FRAME_SNAP, &ifr)) {
-      ife->has_ipx_sn=1;
-      ife->ipxaddr_sn=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if(!ipx_getaddr(ipx_sock, IPX_FRAME_8023, &ifr)) {
-      ife->has_ipx_e3=1;
-      ife->ipxaddr_e3=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if(!ipx_getaddr(ipx_sock, IPX_FRAME_8022, &ifr)) {
-      ife->has_ipx_e2=1;
-      ife->ipxaddr_e2=ifr.ifr_addr;
-    }
-  }
-#endif
-
-  if_getstats(ifname,ife);
-  return(0);
-}
-
-
-static void
-if_print(char *ifname)
-{
-  struct interface ife;
-
-  if (ifname == (char *)NULL) {
-    int i;
-    struct ifconf ifc;
-    struct ifreq *ifr;
-    if (ioctl(skfd, SIOCGIFCOUNT, &i) < 0) {
-      int n = 2, s;
-      ifc.ifc_buf = NULL;
-      do {
-	n *= 2;
-	ifc.ifc_buf = realloc(ifc.ifc_buf, (ifc.ifc_len = s = 
-					    n*sizeof(struct ifreq)));
-	if (ifc.ifc_buf == NULL) {
-	  fprintf(stderr, "Out of memory\n");
-	  exit(1);
-	}
-	
-	if (ioctl(skfd, SIOCGIFCONF, &ifc) < 0) {
-	  perror("SIOCGIFCONF");
-	  return;
-	}
-      } while (ifc.ifc_len == s);
+    if (!ifname) { 
+	for_all_interfaces(do_if_print, &opt_a); 
     } else {
-      ifc.ifc_buf = malloc(ifc.ifc_len = i*sizeof(struct ifreq));
-      if (ifc.ifc_buf == NULL) {
-	fprintf(stderr, "Out of memory.\n");
-	exit(1);
-      }
-      if (ioctl(skfd, SIOCGIFCONF, &ifc) < 0) {
-	perror("SIOCGIFCONF");
-	return;
-      }
+	ife = lookup_interface(ifname); 
+	if (!ife)
+	    fprintf(stderr, _("%s: interface not found.\n"), ifname); 
+	else if (if_fetch(ifname, ife) < 0)
+	    fprintf(stderr, _("%s: error fetching interface information: %s"),
+		    ifname, strerror(errno)); 
+	else 
+	    ife_print(ife);
     }
-    ifr = ifc.ifc_req;
-    for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++) {
-      
-      if (if_fetch(ifr->ifr_name, &ife) < 0) {
-	fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, 
-		      ifconfig_unkn, "%s: unknown interface.\n"),
-		ifr->ifr_name);
-	continue;
-      }
-      
-      if (((ife.flags & IFF_UP) == 0) && !opt_a) continue;
-      ife_print(&ife);
-    }
-    free(ifc.ifc_buf);
-  } else {
-    if (if_fetch(ifname, &ife) < 0)
-      fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, 
-		      ifconfig_unkn, "%s: unknown interface.\n"), ifname);
-    else 
-      ife_print(&ife);
-  }
 }
 
 
@@ -658,9 +371,10 @@ set_flag(char *ifname, short flag)
 
   strcpy(ifr.ifr_name, ifname);
   if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
-    fprintf(stderr, "%s: unknown interface.\n", ifname);
+    fprintf(stderr, _("%s: unknown interface.\n"), ifname);
     return(-1);
   }
+  strcpy(ifr.ifr_name, ifname);
   ifr.ifr_flags |= flag;
   if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
     perror("SIOCSIFFLAGS");
@@ -678,9 +392,10 @@ clr_flag(char *ifname, short flag)
 
   strcpy(ifr.ifr_name, ifname);
   if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
-    fprintf(stderr, "%s: unknown interface.\n", ifname);
+    fprintf(stderr, _("%s: unknown interface.\n"), ifname);
     return -1;
   }
+  strcpy(ifr.ifr_name, ifname);
   ifr.ifr_flags &= ~flag;
   if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
     perror("SIOCSIFFLAGS");
@@ -693,29 +408,41 @@ clr_flag(char *ifname, short flag)
 static void
 usage(void)
 {
-  fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, ifconfig_usage1,
-			      "Usage: ifconfig [-a] [-i] [-v] interface\n"));
-  fprintf(stderr, "                [[family] address]\n");
-#if HAVE_AFINET6
-  fprintf(stderr, "                [add inet6address/prefixlen]\n");
-  fprintf(stderr, "                [del inet6address/prefixlen]\n");
-  fprintf(stderr, "                [tunnel aa.bb.cc.dd]\n");
+  fprintf(stderr, _("Usage: ifconfig [-a] [-i] [-v] interface\n"));
+  fprintf(stderr, _("                [[family] address]\n"));
+  /* XXX: it would be useful to have the add/del syntax even without IPv6.
+	 the 2.1 interface address lists make this natural */ 
+#ifdef HAVE_AFINET6
+  fprintf(stderr, _("                [add address[/prefixlen]]\n"));
+#ifdef SIOCDIFADDR
+  fprintf(stderr, _("                [del address[/prefixlen]]\n"));
+#endif
+  /* XXX the kernel supports tunneling even without ipv6 */ 
+  fprintf(stderr, _("                [tunnel aa.bb.cc.dd]\n"));
 #endif
 #if HAVE_AFINET
-  fprintf(stderr, "                [[-]broadcast [aa.bb.cc.dd]]\n");
-  fprintf(stderr, "                [[-]pointopoint [aa.bb.cc.dd]]\n");
-  fprintf(stderr, "                [netmask aa.bb.cc.dd]\n");
-  fprintf(stderr, "                [dstaddr aa.bb.cc.dd]\n");
+  fprintf(stderr, _("                [[-]broadcast [aa.bb.cc.dd]]\n"));
+  fprintf(stderr, _("                [[-]pointopoint [aa.bb.cc.dd]]\n"));
+  fprintf(stderr, _("                [netmask aa.bb.cc.dd]\n"));
+  fprintf(stderr, _("                [dstaddr aa.bb.cc.dd]\n"));
 #endif
-  fprintf(stderr, "                [hw class address]\n");
-  fprintf(stderr, "                [metric NN] [mtu NN]\n");
-  fprintf(stderr, "                [[-]trailers] [[-]arp]\n");
-  fprintf(stderr, "                [[-]allmulti] [[-]promisc]\n");
-  fprintf(stderr, "                [multicast]\n");
-  fprintf(stderr, "                [mem_start NN] [io_addr NN] [irq NN]\n");
-  fprintf(stderr, "                [media type]\n");
-  fprintf(stderr, "                [up] [down] ...\n");
-  NLS_CATCLOSE(catfd)
+  fprintf(stderr, _("                [hw class address]\n"));
+  fprintf(stderr, _("                [metric NN] [mtu NN]\n"));
+#ifdef SIOCSKEEPALIVE
+  fprintf(stderr, _("                [outfill NN] [keepalive NN]\n"));
+#endif
+  fprintf(stderr, _("                [[-]trailers] [[-]arp]\n"));
+  fprintf(stderr, _("                [[-]allmulti] [[-]promisc]\n"));
+  fprintf(stderr, _("                [multicast]\n"));
+  fprintf(stderr, _("                [mem_start NN] [io_addr NN] [irq NN]\n"));
+  fprintf(stderr, _("                [media type]\n"));
+#ifdef HAVE_TXQUEUELEN
+  fprintf(stderr, _("                [txqueuelen len]\n"));
+#endif
+#ifdef HAVE_DYNAMIC
+  fprintf(stderr, _("				 [[-]dynamic]\n")); 
+#endif
+  fprintf(stderr, _("                [up] [down] ...\n"));
   exit(1);
 }
 
@@ -723,69 +450,24 @@ static void
 version(void)
 {
   fprintf(stderr,"%s\n%s\n",Release,Version);
-  NLS_CATCLOSE(catfd)
   exit(1);
 }
 
-static int sockets_open()
+static int 
+set_netmask(int skfd, struct ifreq *ifr, struct sockaddr *sa)
 {
-#if HAVE_AFINET
-  inet_sock = socket(AF_INET, SOCK_DGRAM, 0);
-#endif
+	int err = 0; 
 
-#if HAVE_AFINET6
-  inet6_sock = socket(AF_INET6, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFIPX 
-  ipx_sock = socket(AF_IPX, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFAX25 
-  ax25_sock = socket(AF_AX25, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_ROSE
-  rose_sock = socket(AF_ROSE, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFATALK
-  ddp_sock = socket(AF_APPLETALK, SOCK_DGRAM, 0);
-#endif
-  
-  /*
-   *	Now pick any (existing) useful socket family for generic queries
-   */
-
-#if HAVE_AFINET
-  if (inet_sock != -1) return inet_sock;
-#endif
-
-#if HAVE_AFINET6
-  if (inet6_sock != -1) return inet6_sock;
-#endif
-
-#if HAVE_AFIPX 
-  if (ipx_sock != -1) return ipx_sock;
-#endif
-
-#if HAVE_AFAX25 
-  if (ax25_sock != -1) return ax25_sock;
-#endif
-
-#if HAVE_AFROSE 
-  if (rose_sock != -1) return rose_sock;
-#endif
-
-#if HAVE_AFATALK
-  if (ddp_sock != -1) return ddp_sock;
-#endif
-
-  /* We have no address families.  */
-  fprintf(stderr, "No usable address families found.\n");
-  return -1;
+	memcpy((char *) &ifr->ifr_netmask, (char *) sa,
+		   sizeof(struct sockaddr));
+	if (ioctl(skfd, SIOCSIFNETMASK, ifr) < 0) {
+		fprintf(stderr, "SIOCSIFNETMASK: %s\n",
+				strerror(errno));
+		err = 1;
+	}
+	return 0;
 }
-	
+
 int
 main(int argc, char **argv)
 {
@@ -794,7 +476,7 @@ main(int argc, char **argv)
   struct aftype *ap;
   struct hwtype *hw;
   struct ifreq ifr;
-  int goterr = 0;
+  int goterr = 0, didnetmask = 0;
   char **spp;
 #if HAVE_AFINET6
   extern struct aftype inet6_aftype;
@@ -804,15 +486,14 @@ main(int argc, char **argv)
   char *cp;
 #endif
   
-#if NLS
-  setlocale (LC_MESSAGES, "");
-  catfd = catopen ("nettools", MCLoadBySet);
+#if I18N
+  bindtextdomain("net-tools", "/usr/share/locale");
+  textdomain("net-tools");
 #endif
 
   /* Create a channel to the NET kernel. */
-  if ((skfd = sockets_open()) < 0) {
+  if ((skfd = sockets_open(0)) < 0) {
     perror("socket");
-    NLS_CATCLOSE(catfd)
     exit(1);
   }
 
@@ -837,7 +518,6 @@ main(int argc, char **argv)
   if (argc == 0) {
     if_print((char *)NULL);
     (void) close(skfd);
-    NLS_CATCLOSE(catfd)
     exit(0);
   }
 
@@ -847,17 +527,24 @@ main(int argc, char **argv)
   if (*spp == (char *)NULL) {
     if_print(ifr.ifr_name);
     (void) close(skfd);
-    NLS_CATCLOSE(catfd)
     exit(0);
   }
 
   /* The next argument is either an address family name, or an option. */
   if ((ap = get_aftype(*spp)) == NULL) 
     ap = get_aftype("inet");
-  else 
-    spp++;
+  else { 
+	  /* XXX: should print the current setup if no args left, but only 
+		 for this family */  
+	  spp++;
+  }
   addr_family = ap->af;
 
+  if (sockets_open(addr_family) < 0) { 
+	  perror("family socket");  
+	  exit(1); 
+  }
+  
   /* Process the remaining arguments. */
   while (*spp != (char *)NULL) {
     if (!strcmp(*spp, "arp")) {
@@ -872,7 +559,8 @@ main(int argc, char **argv)
       continue;
     }
 
-    if (!strcmp(*spp, "media") || !strcmp(*spp, "port") || !strcmp(*spp, "link")) {
+#ifdef IFF_PORTSEL
+    if (!strcmp(*spp, "media") || !strcmp(*spp, "port")) {
       if (*++spp == NULL) usage();
       if (!strcasecmp(*spp, "auto")) {
 	goterr |= set_flag(ifr.ifr_name, IFF_AUTOMEDIA);
@@ -893,7 +581,7 @@ main(int argc, char **argv)
 	}
 	spp++;
 	if (newport == -1) {
-	  fprintf(stderr, "Unknown media type.\n");
+	  fprintf(stderr, _("Unknown media type.\n"));
 	  goterr = 1;
 	} else {
 	  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
@@ -909,6 +597,7 @@ main(int argc, char **argv)
       }
       continue;
     }
+#endif
     
     if (!strcmp(*spp, "trailers")) {
       goterr |= clr_flag(ifr.ifr_name, IFF_NOTRAILERS);
@@ -963,13 +652,26 @@ main(int argc, char **argv)
       spp++;
       continue;
     }
-    
+
     if (!strcmp(*spp, "down")) {
       goterr |= clr_flag(ifr.ifr_name, IFF_UP);
       spp++;
       continue;
     }
 
+#ifdef HAVE_DYNAMIC
+	if (!strcmp(*spp, "dynamic")) { 
+	  goterr |= set_flag(ifr.ifr_name, IFF_DYNAMIC); 
+	  spp++;
+	  continue;
+	}
+	if (!strcmp(*spp, "-dynamic")) {
+	  goterr |= clr_flag(ifr.ifr_name, IFF_DYNAMIC);
+	  spp++;
+	  continue;
+	}
+#endif
+
     if (!strcmp(*spp, "metric")) {
       if (*++spp == NULL) usage();
       ifr.ifr_metric = atoi(*spp);
@@ -991,6 +693,32 @@ main(int argc, char **argv)
       spp++;
       continue;
     }
+
+#ifdef SIOCSKEEPALIVE
+    if (!strcmp(*spp, "keepalive")) {
+      if (*++spp == NULL) usage();
+      ifr.ifr_data = (caddr_t)atoi(*spp);
+      if (ioctl(skfd, SIOCSKEEPALIVE, &ifr) < 0) {
+       fprintf(stderr, "SIOCSKEEPALIVE: %s\n", strerror(errno));
+       goterr = 1;
+      }
+      spp++;
+      continue;
+    }
+#endif
+
+#ifdef SIOCSOUTFILL
+    if (!strcmp(*spp, "outfill")) {
+      if (*++spp == NULL) usage();
+      ifr.ifr_data = (caddr_t)atoi(*spp);
+      if (ioctl(skfd, SIOCSOUTFILL, &ifr) < 0) {
+       fprintf(stderr, "SIOCSOUTFILL: %s\n", strerror(errno));
+       goterr = 1;
+      }
+      spp++;
+      continue;
+    }
+#endif
     
     if (!strcmp(*spp, "-broadcast")) {
       goterr |= clr_flag(ifr.ifr_name, IFF_BROADCAST);
@@ -1000,7 +728,8 @@ main(int argc, char **argv)
     
     if (!strcmp(*spp, "broadcast")) {
       if (*++spp != NULL ) {
-	strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
 	if (ap->input(0, host, &sa) < 0) {
 	  ap->herror(host);
 	  goterr = 1;
@@ -1022,7 +751,8 @@ main(int argc, char **argv)
     
     if (!strcmp(*spp, "dstaddr")) {
       if (*++spp == NULL) usage();
-      strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
       if (ap->input(0, host, &sa) < 0) {
 	ap->herror(host);
 	goterr = 1;
@@ -1041,25 +771,34 @@ main(int argc, char **argv)
     }
     
     if (!strcmp(*spp, "netmask")) {
-      if (*++spp == NULL) usage();
-      strcpy(host, *spp);
+      if (*++spp == NULL || didnetmask) usage();
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
       if (ap->input(0, host, &sa) < 0) {
 	ap->herror(host);
 	goterr = 1;
 	spp++;
 	continue;
       }
-      memcpy((char *) &ifr.ifr_netmask, (char *) &sa,
-	     sizeof(struct sockaddr));
-      if (ioctl(skfd, SIOCSIFNETMASK, &ifr) < 0) {
-	fprintf(stderr, "SIOCSIFNETMASK: %s\n",
-		strerror(errno));
+	  didnetmask++;
+	  goterr = set_netmask(skfd, &ifr, &sa);
+      spp++;
+      continue;
+    }
+
+#ifdef HAVE_TXQUEUELEN
+    if (!strcmp(*spp, "txqueuelen")) {
+      if (*++spp == NULL) usage();
+      ifr.ifr_qlen = strtoul(*spp, NULL, 0);
+      if (ioctl(skfd, SIOCSIFTXQLEN, &ifr) < 0) {
+	fprintf(stderr, "SIOCSIFTXQLEN: %s\n", strerror(errno));
 	goterr = 1;
       }
       spp++;
       continue;
     }
-    
+#endif
+
     if (!strcmp(*spp, "mem_start")) {
       if (*++spp == NULL) usage();
       if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
@@ -1114,7 +853,8 @@ main(int argc, char **argv)
     if (!strcmp(*spp, "pointopoint")) {
       if (*(spp+1) != NULL) {
 	spp++;
-	strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
 	if (ap->input(0, host, &sa)) {
 	  ap->herror(host);
 	  goterr = 1;
@@ -1138,9 +878,10 @@ main(int argc, char **argv)
       if (*++spp == NULL) usage();
       if ((hw = get_hwtype(*spp)) == NULL) usage();
       if (*++spp == NULL) usage();
-      strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
       if (hw->input(host, &sa) < 0) {
-	fprintf(stderr, "%s: invalid %s address.\n", host, hw->name);
+	fprintf(stderr, _("%s: invalid %s address.\n"), host, hw->name);
 	goterr = 1;
 	spp++;
 	continue;
@@ -1166,8 +907,9 @@ main(int argc, char **argv)
       } else {
 	prefix_len = 0;
       }
-      strcpy(host, *spp);
-      if (inet6_aftype.input(1, host, (struct sockaddr *)&sa6) < 0) {
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
+     if (inet6_aftype.input(1, host, (struct sockaddr *)&sa6) < 0) {
 	inet6_aftype.herror(host);
 	goterr = 1;
 	spp++;
@@ -1203,7 +945,8 @@ main(int argc, char **argv)
       } else {
 	prefix_len = 0;
       }
-      strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
       if (inet6_aftype.input(1, host, (struct sockaddr *)&sa6) < 0) {
 	inet6_aftype.herror(host);
 	goterr = 1;
@@ -1222,11 +965,15 @@ main(int argc, char **argv)
 
       ifr6.ifr6_ifindex = ifr.ifr_ifindex;
       ifr6.ifr6_prefixlen = prefix_len;
+#ifdef SIOCDIFADDR
       if (ioctl(inet6_sock, SIOCDIFADDR, &ifr6) < 0) {
 	fprintf(stderr, "SIOCDIFADDR: %s\n",
 		strerror(errno));
 	goterr = 1;
       }
+#else
+      fprintf(stderr, _("Address deletion not supported on this system.\n"));
+#endif
       spp++;
       continue;
     }
@@ -1240,7 +987,8 @@ main(int argc, char **argv)
       } else {
 	prefix_len = 0;
       }
-      strcpy(host, *spp);
+		  host[(sizeof host)-1] = 0; 
+		  strncpy(host, *spp, (sizeof host)-1);
       if (inet6_aftype.input(1, host, (struct sockaddr *)&sa6) < 0) {
 	inet6_aftype.herror(host);
 	goterr = 1;
@@ -1271,16 +1019,50 @@ main(int argc, char **argv)
 #endif
     
     /* If the next argument is a valid hostname, assume OK. */
-    strcpy(host, *spp);
+	host[(sizeof host)-1] = '\0'; 
+    strncpy(host, *spp, (sizeof host)-1);
+
+	/* FIXME: sa is too small for INET6 addresses, inet6 should use that too, 
+	 		  broadcast is unexpected */ 
+	if (ap->getmask) { 
+		switch (ap->getmask(host, &sa, NULL)) {
+		case -1: usage(); break; 
+		case 1:  
+			if (didnetmask) usage(); 
+
+			goterr = set_netmask(skfd, &ifr, &sa); 
+			didnetmask++; 
+			break;
+		}
+	}
+
     if (ap->input(0, host, &sa) < 0) {
       ap->herror(host);
       usage();
     }
 
     memcpy((char *) &ifr.ifr_addr, (char *) &sa, sizeof(struct sockaddr));
-    if (ioctl(skfd, SIOCSIFADDR, &ifr) < 0) {
-      fprintf(stderr, "SIOCSIFADDR: %s\n", strerror(errno));
-      goterr = 1;
+    {
+      int r = 0; /* to shut gcc up */  
+      switch (ap->af) {
+#if HAVE_AFINET
+      case AF_INET:
+	r = ioctl(inet_sock, SIOCSIFADDR, &ifr);
+	break;
+#endif
+#if HAVE_AFECONET
+      case AF_ECONET:
+	r = ioctl(ec_sock, SIOCSIFADDR, &ifr);
+	break;
+#endif
+      default:
+	printf(_("Don't know how to set addresses for this family.\n"));
+	exit(1);
+      }
+      if (r < 0) {
+	fprintf(stderr, "SIOCSIFADDR: %s\n", strerror(errno));
+	goterr = 1;
+      }
     }
     goterr |= set_flag(ifr.ifr_name, (IFF_UP | IFF_RUNNING));
     spp++;
@@ -1289,6 +1071,5 @@ main(int argc, char **argv)
   /* Close the socket. */
   (void) close(skfd);
 
-  NLS_CATCLOSE(catfd)
   return(goterr);
 }
