@@ -8,7 +8,7 @@
  *		NET-3 Networking Distribution for the LINUX operating
  *		system. (net-tools, net-drivers)
  *
- * Version:	lib/net-support.h 1.35 (1997-09-25)
+ * Version:	lib/net-support.h 1.34 (1996-04-13)
  *
  * Maintainer:	Bernd 'eckes' Eckenfels, <net-tools@lina.inka.de>
  *
@@ -22,7 +22,6 @@
  *960219 {1.32} Bernd Eckenfels:	type for ap->input()
  *960322 {1.33} Bernd Eckenfels:	activate_ld and const in get_hwtype
  *960413 {1.34} Bernd Eckenfels:	new RTACTION suport
- *970925 {1.35} Bernd Eckenfels:	Terry's Rose support
  *
  *		This program is free software; you can redistribute it
  *		and/or  modify it under  the terms of  the GNU General
@@ -44,6 +43,10 @@ struct aftype {
   void		(*herror)	(char *text);
   int		(*rprint)	(int options);
   int 		(*rinput)	(int typ, int ext, char **argv);
+
+  /* may modify src */ 
+  int		(*getmask)  (char *src, struct sockaddr *mask, char *name); 	
+
 };
 
 
@@ -95,7 +98,6 @@ extern int	DDP_rprint(int options);
 extern int	IPX_rprint(int options);
 extern int	NETROM_rprint(int options);
 extern int	AX25_rprint(int options);
-extern int	ROSE_rprint(int options);
 
 extern int	INET_rinput(int action, int flags, char **argv);
 extern int	INET6_rinput(int action, int flags, char **argv);
@@ -103,7 +105,6 @@ extern int	DDP_rinput(int action, int flags, char **argv);
 extern int	IPX_rinput(int action, int flags, char **argv);
 extern int	NETROM_rinput(int action, int flags, char **argv);
 extern int	AX25_rinput(int action, int flags, char **argv);
-extern int	ROSE_rinput(int action, int flags, char **argv);
 
 int	aftrans_opt	(const char *arg);
 void	aftrans_def	(char *tool, char *argv0, char *dflt);
@@ -113,7 +114,6 @@ char * get_sname(int socknumber, char *proto, int numeric);
 extern int flag_unx;
 extern int flag_ipx;
 extern int flag_ax25;
-extern int flag_rose;
 extern int flag_ddp;
 extern int flag_netrom;
 extern int flag_inet;
@@ -129,19 +129,19 @@ extern char afname[];
 	{"netrom",	0,	0,	1}, \
 	{"inet",	0,	0,	1}, \
 	{"ddp",		0,	0,	1}, \
-        {"rose",        0,      0,      1}, \
 	{"unix",	0,	0,	1}, \
 	{"tcpip",	0,	0,	1}
-#define AFTRANS_CNT 10
+#define AFTRANS_CNT 9
 
 #define EINTERN(file, text) fprintf(stderr, \
-	"%s: Internal Error `%s'.\nContact: %s\n",file,text,Maintainer);
+	"%s: Internal Error `%s'.\n",file,text);
 
-#define ENOSUPP(A,B)	fprintf(stderr, NLS_CATGETS(catfd, libSet, lib_nofeature, \
-				"%s: feature `%s' not supported.\nPlease recompile `net-tools' with newer kernel source or full configuration.\n"),A,B)
+#define ENOSUPP(A,B)	fprintf(stderr,\
+                                _("%s: feature `%s' not supported.\n" \
+				  "Please recompile `net-tools' with "\
+				  "newer kernel source or full configuration.\n"),A,B)
 
-#define ESYSNOT(A,B)	fprintf(stderr, NLS_CATGETS(catfd, libSet, lib_sysnot, \
-				"%s: no support for `%s' on this system.\n"),A,B)
+#define ESYSNOT(A,B)	fprintf(stderr, _("%s: no support for `%s' on this system.\n"),A,B)
 
 #define E_NOTFOUND	8
 #define E_SOCK		7
